--
-- Claas Axion850
-- Specialization for Claas Axion850
--
-- @author  Repi, Sven777b, Manuel Leithner
-- @date  29/07/2011
--
-- frei verwendbar - keine erlaubnis ntig  |  free for use - no permission needed.
-- Modifikationen erst nach Rcksprache!    |  modifications only with my permission.
--
-- Hinweis: das kopieren von Sourcecode ist strafbar nach UrhG 2.1

Axion850Event = {};
Axion850Event_mt = Class(Axion850Event, Event);

InitEventClass(Axion850Event, "Axion850Event");

function Axion850Event:emptyNew()
    local self = Event:new(Axion850Event_mt);
    self.className="Axion850Event";
    return self;
end;

function Axion850Event:new(vehicle, level, percentage, doRestoreLimit)
    local self = Axion850Event:emptyNew()
    self.vehicle = vehicle;
	self.level = level;
	self.percentage = percentage;
	self.doRestoreLimit = doRestoreLimit;
    return self;
end;

function Axion850Event:readStream(streamId, connection)
    local id = streamReadInt32(streamId);
	self.level = streamReadInt8(streamId);
	self.percentage = streamReadInt8(streamId);
	self.doRestoreLimit = streamReadBool(streamId);
    self.vehicle = networkGetObject(id);
    self:run(connection);
end;

function Axion850Event:writeStream(streamId, connection)
    streamWriteInt32(streamId, networkGetObjectId(self.vehicle));	
	streamWriteInt8(streamId, self.level);
	streamWriteInt8(streamId, self.percentage);
	streamWriteBool(streamId, self.doRestoreLimit);
end;

function Axion850Event:run(connection)
	self.vehicle:setNewLimit(self.level, self.percentage, self.doRestoreLimit, true);
	if not connection:getIsServer() then
		--g_server:broadcastEvent(Axion850Event:new(self.vehicle, self.level, self.percentage, self.doRestoreLimit), nil, connection, self.object);
	end;
end;

function Axion850Event.sendEvent(vehicle, level, percentage, doRestoreLimit, noEventSend)
	if noEventSend == nil or noEventSend == false then
		if g_server ~= nil then
			-- no need to broadcast the event: only the server need the information
			--g_server:broadcastEvent(Axion850Event:new(vehicle, level, percentage, doRestoreLimit), nil, nil, vehicle);
		else
			g_client:getServerConnection():sendEvent(Axion850Event:new(vehicle, level, percentage, doRestoreLimit));
		end;
	end;
end;

Axion850AnimEvent = {};
Axion850AnimEvent_mt = Class(Axion850AnimEvent, Event);
InitEventClass(Axion850AnimEvent, "Axion850AnimEvent");

function Axion850AnimEvent:emptyNew()
    local self = Event:new(Axion850AnimEvent_mt);
    self.className="Axion850AnimEvent";
    return self;
end;

function Axion850AnimEvent:new(object, id, state)
		local self = Axion850AnimEvent:emptyNew()
		self.object = object;
		self.id = id;
		self.state = state;
		return self;
end;

function Axion850AnimEvent:readStream(streamId, connection)
    self.object = networkGetObject(streamReadInt32(streamId));
		self.id  = streamReadInt8(streamId);
    self.state = streamReadBool(streamId);
    self:run(connection);
end;

function Axion850AnimEvent:writeStream(streamId, connection)
    		streamWriteInt32(streamId, networkGetObjectId(self.object));
		streamWriteInt8(streamId, self.id);
		streamWriteBool(streamId, self.state);
end;

function Axion850AnimEvent:run(connection)
	self.object:toggle(self.id,self.state, true);
  if not connection:getIsServer() then
      g_server:broadcastEvent(Axion850AnimEvent:new(self.object, self.id, self.state), nil, connection, self.object);
  end;	
end;

function Axion850AnimEvent.sendEvent(vehicle, id, state, nes)
	if nes == nil or nes == false then
		if g_server ~= nil then
			g_server:broadcastEvent(Axion850AnimEvent:new(vehicle, id, state), nil, nil, vehicle);
		else
			g_client:getServerConnection():sendEvent(Axion850AnimEvent:new(vehicle, id, state));
		end;
	end;
end;

Axion850WheelsEvent = {};
Axion850WheelsEvent_mt = Class(Axion850WheelsEvent, Event);

InitEventClass(Axion850WheelsEvent, "Axion850WheelsEvent");

Axion850WheelsEventType = {
	TwinWheels = 1,
}

function Axion850WheelsEvent:emptyNew()
    local self = Event:new(Axion850WheelsEvent_mt);
    self.className="Axion850WheelsEvent";
    return self;
end;

function Axion850WheelsEvent:new(object, eventType, data)
    local self = Axion850WheelsEvent:emptyNew()
    self.object = object;
	self.eventType = eventType;
	self.data = data;
    return self;
end;

function Axion850WheelsEvent:readStream(streamId, connection)
    local id = streamReadInt32(streamId);
	self.object = networkGetObject(id);
	self.eventType = streamReadUInt8(streamId);
	if self.eventType == Axion850WheelsEventType.TwinWheels then
		self.data = streamReadBool(streamId);
		self.object:settwinWheels(self.data, true);
	end;
    if not connection:getIsServer() then
        g_server:broadcastEvent(Axion850WheelsEvent:new(self.object, self.eventType, self.data), nil, connection, self.object);
    end;
end;

function Axion850WheelsEvent:writeStream(streamId, connection)
    streamWriteInt32(streamId, networkGetObjectId(self.object));
	streamWriteUInt8(streamId, self.eventType);
	streamWriteBool(streamId, self.data);
end;

function Axion850WheelsEvent.sendEvent(vehicle, eventType, data)
	if g_server ~= nil then
		g_server:broadcastEvent(Axion850WheelsEvent:new(vehicle, eventType, data), nil, nil, vehicle);
	else
		g_client:getServerConnection():sendEvent(Axion850WheelsEvent:new(vehicle, eventType, data));
	end;
end;

Axion850LightStateEvent = {};
Axion850LightStateEvent_mt = Class(Axion850LightStateEvent, Event);

InitEventClass(Axion850LightStateEvent, "Axion850LightStateEvent");

function Axion850LightStateEvent:emptyNew()
    local self = Event:new(Axion850LightStateEvent_mt);
    self.className="Axion850LightStateEvent";
    return self;
end;

function Axion850LightStateEvent:new(object, light, state)
		local self = Axion850LightStateEvent:emptyNew()
		self.object = object;
		self.light = light;
		self.state = state;
		return self;
end;

function Axion850LightStateEvent:readStream(streamId, connection)
    self.object = networkGetObject(streamReadInt32(streamId));
		self.light  = streamReadString(streamId);
    self.state = streamReadBool(streamId);
    self:run(connection);
end;

function Axion850LightStateEvent:writeStream(streamId, connection)
    streamWriteInt32(streamId, networkGetObjectId(self.object));
		streamWriteString(streamId, self.light);
		streamWriteBool(streamId, self.state);
end;

function Axion850LightStateEvent:run(connection)
	self.object:setState(self.light,self.state, true);
  if not connection:getIsServer() then
      g_server:broadcastEvent(Axion850LightStateEvent:new(self.object, self.light, self.state), nil, connection, self.object);
  end;	
end;

function Axion850LightStateEvent.sendEvent(vehicle, light, state, noEventSend)
	if noEventSend == nil or noEventSend == false then
		if g_server ~= nil then
			g_server:broadcastEvent(Axion850LightStateEvent:new(vehicle, light, state), nil, nil, vehicle);
		else
			g_client:getServerConnection():sendEvent(Axion850LightStateEvent:new(vehicle, light, state));
		end;
	end;
end;

-----------------------------------------------------------------------------------------------------

Axion850 = {};
function Axion850.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Motorized, specializations) and SpecializationUtil.hasSpecialization(Steerable, specializations);
end;

function Axion850:load(xmlFile)

         --Dynamic anfang--
	self.exhaustingSystem = {};	
	self.exhaustingSystem.cap = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.dynamicExhaustingSystem#cap"));
	self.exhaustingSystem.maxRot = Utils.degToRad(Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.dynamicExhaustingSystem#maxRot"),0));
	self.exhaustingSystem.maxRpm = self.motor:getMaxRpm();
	
	local startSequence = AnimCurve:new(linearInterpolator4);
	local i=0;
	while true do
		local path = string.format("vehicle.dynamicExhaustingSystem.startSequence.key(%d)", i);
		local timeStamp = getXMLFloat(xmlFile, path .. "#time");
		if timeStamp == nil then
			break;
		end;
		local r,g,b,alpha = Utils.getVectorFromString(getXMLString(xmlFile, path .. "#value"));
		startSequence:addKeyframe({x=r, y=g, z=b, w=alpha, time=timeStamp})		
		i = i + 1;
	end;
	self.exhaustingSystem.minAlpha = math.min(math.abs(Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.dynamicExhaustingSystem#minAlpha"),0)),1);
	self.exhaustingSystem.maxAlpha = math.min(math.abs(Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.dynamicExhaustingSystem#maxAlpha"),1)),1);
	local x1,y1,z1,w1 = startSequence:get(1.0);
	self.exhaustingSystem.startSequence = startSequence;
	self.exhaustingSystem.lastValue = {x=x1,y=y1,z=z1,w=w1};
	self.exhaustingSystem.param = getXMLString(xmlFile, "vehicle.dynamicExhaustingSystem#param");
	self.exhaustingSystem.offset = 0;
	self.exhaustingSystem.deltaTime = 0;
        --Dynamic ende--

        --ESLimiter anfang--
	self.setNewLimit = SpecializationUtil.callSpecializationsFunction("setNewLimit");

	self.ESLimiter = {};
	self.ESLimiter.xPos = Utils.getNoNil(getXMLFloat(xmlFile,  "vehicle.ESLimiter#posX"), 0.7543);
	self.ESLimiter.yPos = Utils.getNoNil(getXMLFloat(xmlFile,  "vehicle.ESLimiter#posY"), 0.1715);
	self.ESLimiter.maxRPM = {};
	self.ESLimiter.percentage = {};
	for k,v in pairs(self.motor.maxRpm) do
		self.ESLimiter.maxRPM[k] = self.motor.maxRpm[3];
		local percentage = string.format("%d",(self.motor.maxRpm[k]* 100 / self.motor.maxRpm[3]));
		self.ESLimiter.percentage[k] = tonumber(percentage); -- percentage
	end;

	self.eslOverlay = Overlay:new("hudESLOverlay", Utils.getFilename("Textur/ESLimiter_hud.png", self.baseDirectory), self.ESLimiter.xPos, self.ESLimiter.yPos, 0.2371, 0.039525);
	self.lastButtonDown = nil;
	self.isLimiterActive = false;
        --ESLimiter ende--

        --Wiper anfang--
    local wiperAnimRootNode = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.wiper#rootNode"));
    self.wiperAnimCharSet = 0;
    if wiperAnimRootNode ~= nil and wiperAnimRootNode ~= 0 then
        self.wiperAnimCharSet = getAnimCharacterSet(wiperAnimRootNode);
        if self.wiperAnimCharSet ~= 0 then
            local clip = getAnimClipIndex(self.wiperAnimCharSet, getXMLString(xmlFile, "vehicle.wiper#clip"));
            assignAnimTrackClip(self.wiperAnimCharSet, 0, clip);
            setAnimTrackLoopState(self.wiperAnimCharSet, 0, true);
            local wiperAnimSpeedScale = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.wiper#speedScale"), 1);
			setAnimTrackSpeedScale(self.wiperAnimCharSet, 0, wiperAnimSpeedScale);
            self.wiperAnimDuration = getAnimClipDuration(self.wiperAnimCharSet, clip);
        end;
    end;
	self.isWiperActive = false;
	self.finishWiper = true;
        --Wiper ende--

        --Oper anfang--
	self.addOperatingTime = SpecializationUtil.callSpecializationsFunction("addOperatingTime");
	self.operatingTime = 0;	
	self.ohX = Utils.getNoNil(getXMLFloat(xmlFile,  "vehicle.operatingHours#posX"), 0.7543);
	self.ohY = Utils.getNoNil(getXMLFloat(xmlFile,  "vehicle.operatingHours#posY"), 0.1715);
	self.yOffset = 0.0077;
	self.ohOverlay = Overlay:new("hudOHOverlay", Utils.getFilename("Textur/OperatingHours_hud.png", self.baseDirectory), self.ohX, self.ohY, 0.11855, 0.039525);
        --Oper ende--

        --Power anfang--
	local i=0;
    while true do
        local baseName = string.format("vehicle.attacherJoints.attacherJoint(%d)", i);
        local index = getXMLString(xmlFile, baseName.. "#index");
        if index == nil then
            break;
        end;		
		local joint = self.attacherJoints[i+1];
		local shaftAttacher = Utils.indexToObject(self.components, getXMLString(xmlFile, baseName .. "#shaft"));
		if shaftAttacher ~= nil then
			joint.powerShaftAttacher = shaftAttacher;
		end;
		i = i + 1;
	end;

	local attacherJointCount = i;
	i = 0;
    while true do
        local baseName = string.format("vehicle.trailerAttacherJoints.trailerAttacherJoint(%d)", i);
        local index = getXMLString(xmlFile, baseName.. "#index");
        if index == nil then
            break;
        end;
		
		local joint = self.attacherJoints[attacherJointCount + 1 + i];
		local shaftAttacher = Utils.indexToObject(self.components, getXMLString(xmlFile, baseName .. "#shaft"));
		if shaftAttacher ~= nil then
			joint.powerShaftAttacher = shaftAttacher;
		end;
        i = i+1;
    end;
      --Power ende--

	self.showHelp = false;

      --Lightbar anfang--
	local lampscount = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.lamps#count"), 0);
	self.lamps = {};
	if lampscount ~= 0 then
		for i=1, lampscount do
			local lampnumber = string.format("vehicle.lamps.lamp%d", i);	
			self.lamps[i] = Utils.indexToObject(self.components, getXMLString(xmlFile, lampnumber .. "#index"));	
		end;	
	end;
      --Lightbar ende--

      --toggle anfang--
	self.toggle = SpecializationUtil.callSpecializationsFunction("toggle");
	self.animParts = {};
	local c = 0;
	while true do
		local baseString = string.format("vehicle.animParts.animPart(%d)#",c);
		local index = getXMLString(xmlFile,baseString.."index");
		local id = getXMLString(xmlFile,baseString.."id");
		if index == nil or id == nil or InputBinding[id] == nil then
			break;
		end;
		local part = {};
		part.index = Utils.indexToObject(self.components, index);
		part.id = id;
		
		local rotMin = getXMLString(xmlFile,baseString.."rotMin");
		local rotMax = getXMLString(xmlFile,baseString.."rotMax");
		if rotMin ~= nil and rotMax ~= nil then
			local x,y,z = Utils.getVectorFromString(rotMin);
			part.rotMin = {math.rad(Utils.getNoNil(x,0)),math.rad(Utils.getNoNil(y,0)),math.rad(Utils.getNoNil(z,0))};
			local x,y,z = Utils.getVectorFromString(rotMax);
			part.rotMax = {math.rad(Utils.getNoNil(x,0)),math.rad(Utils.getNoNil(y,0)),math.rad(Utils.getNoNil(z,0))};
		end;
		
		local transMin = getXMLString(xmlFile,baseString.."transMin");
		local transMax = getXMLString(xmlFile,baseString.."transMax");
		if transMin ~= nil and transMax ~= nil then
			local x,y,z = Utils.getVectorFromString(transMin);
			part.transMin = {Utils.getNoNil(x,0),Utils.getNoNil(y,0),Utils.getNoNil(z,0)};
			local x,y,z = Utils.getVectorFromString(transMax);
			part.transMax = {Utils.getNoNil(x,0),Utils.getNoNil(y,0),Utils.getNoNil(z,0)};
		end;
		
		local scaleMin = getXMLString(xmlFile,baseString.."scaleMin");
		local scaleMax = getXMLString(xmlFile,baseString.."scaleMax");
		if scaleMin ~= nil and scaleMax ~= nil then
			local x,y,z = Utils.getVectorFromString(scaleMin);
			part.scaleMin = {Utils.getNoNil(x,1),Utils.getNoNil(y,1),Utils.getNoNil(z,1)};
			local x,y,z = Utils.getVectorFromString(scaleMax);
			part.scaleMax = {Utils.getNoNil(x,1),Utils.getNoNil(y,1),Utils.getNoNil(z,1)};
		end;
		
		part.moveTime = Utils.getNoNil(getXMLFloat(xmlFile,baseString.."moveTime"),1)*1000;
		part.helpText = Utils.getNoNil(getXMLBool(xmlFile, baseString.."showHelp"),false);
		part.autoReturn = Utils.getNoNil(getXMLBool(xmlFile, baseString.."autoReturn"),false);
		part.toMax = false;
		part.move = false;
		table.insert(self.animParts,part);
		c=c+1;
	end;
       --toggle ende--

       --BGA anfang--
         self.bunkerSiloCompactingScale = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.bunkerSiloCompacter#compactingScale"), 1);
         self.bunkerSiloCompactingRefNode = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.bunkerSiloCompacter#refNode"));
       --BGA ende--

       --Twin anfang--
      self.settwinWheels = SpecializationUtil.callSpecializationsFunction("settwinWheels");
	self.numTwinWheels = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.twinWheels#count"), 0);
          self.twinWheels = {};
            for i=1, self.numTwinWheels do
              local twinWheelnamei = string.format("vehicle.twinWheels.twinWheel%d", i);
          self.twinWheels[i] = Utils.indexToObject(self.components, getXMLString(xmlFile, twinWheelnamei .. "#index"));
          setVisibility(self.twinWheels[i], false);
      end;
      self.twinWheelsActive= false;
      --Twin ende--

      --BEL3 anfang--
	local error = "Error: Beleuchtung V3 - %s (%s)";
	self.setState = SpecializationUtil.callSpecializationsFunction("setState");
	
	self.B3 = {}; 
	local i = 0; local fin = false;
	while true do
		while true do
			local on = string.format("vehicle.lightsaddon.light(%d)",i);
			local lt = getXMLString(xmlFile, on .. "#type");
			if lt == nil then fin=true; break; end;
			local is = getXMLString(xmlFile, on .. "#index");
			if is == nil then
				print(string.format(error,"kein Index definiert",on));
				i=i+1;
				break;
			end;
			local index = Utils.indexToObject(self.components, is);
			if index == nil or index == 0 then
				print(string.format(error,"Index "..is.." nicht gefunden",on));
				i=i+1;
				break;
			end;
			
			local tl = {};
			tl.na = getXMLString(xmlFile, on .. "#inputName");
			tl.index = index;
			tl.so = Utils.getNoNil(getXMLBool(xmlFile, on .. "#stayOn"),false);
			tl.a = false;
			tl.ht = getXMLString(xmlFile, on .. "#helptext");
			local rl = getXMLString(xmlFile,on.."#real");
			if rl ~= nil then
				tl.rl = Utils.indexToObject(self.components, rl);
			end;
			local fl = getXMLString(xmlFile,on.."#beam");
			if fl ~= nil then
				tl.fl = Utils.indexToObject(self.components, fl);
			end;
			
			if self.B3[lt] == nil then
				self.B3[lt] = {};
			end;
			table.insert(self.B3[lt],tl);
			setVisibility(tl.index, false);
			if tl.fl ~= nil then
				setVisibility(tl.fl, false);
			end;
			if tl.rl ~= nil then
				setVisibility(tl.rl, false);
			end;
			i=i+1;
		end;
		if fin then break; end;
	end;
	self.B3.lso = Utils.getNoNil(getXMLBool(xmlFile,"vehicle.lightsaddon#lightsStayOn"),false);
	self.B3.md = Utils.getNoNil(getXMLInt(xmlFile,"vehicle.lightsaddon#modDirection"),1);
	self.B3.bspd   = Utils.getNoNil(getXMLFloat(xmlFile,"vehicle.lightsaddon#blinkSpeed"),0.75) * 1000;
	
	self.B3.atv = nil;
	self.B3.setLA = false;
	self.B3.noRL = false;
	self.B3.wl = false;
	self.B3.blt = 0;
	self.B3.bls = false;
	self.B3.left = false;
	self.deactivateLightsOnLeave = not self.B3.lso;
	if SpecializationUtil.hasSpecialization(Steerable, self.specializations) then
		self.B3.iSt = true;
	else
		self.B3.iSt = false;
	end;
        --BEL3 ende--
		
		--Hydraulic anfang--
    self.backHydraulic = {};
	self.backHydraulic.rootArm = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.backHydraulic#rootArm"));
	self.backHydraulic.bottomArm = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.backHydraulic#bottomArm"));
	self.frontAttacherJoint = {};
	self.frontAttacherJoint.node = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.trailerAttacherJoints#front"));
	    --Hydraulic ende--

end;

function Axion850:delete()
end;

function Axion850:mouseEvent(posX, posY, isDown, isUp, button)
end;

function Axion850:keyEvent(unicode, sym, modifier, isDown)
end;

function Axion850:readStream(streamId, connection)

        --ESLimiter anfang--
	for level,_ in pairs(self.ESLimiter.percentage) do
		local percentage = streamReadInt8(streamId);
		self:setNewLimit(level, percentage, false, true);
	end;
        --ESLimiter ende--

        --Oper anfang--
	self:addOperatingTime(streamReadFloat32(streamId), true);
        --Oper ende--

        --toggle anfang--
	for nr,part in ipairs(self.animParts) do
		local state = streamReadBool(streamId);
		self:toggle(nr,state,true);
	end;
        --toggle ende--

        --Twin anfang--
	self:settwinWheels(streamReadBool(streamId), true);
        --Twin ende--

        --BEL3 anfang--
	if self.B3.work ~= nil then
		for i=1,table.getn(self.B3.work) do
			self:setState("work:"..i,streamReadBool(streamId),true);
		end;
	end;
	if self.B3.highbeam ~= nil then
		self:setState("highbeam",streamReadBool(streamId),true);
	end;
	if self.B3.dirLeft ~= nil and self.B3.dirRight ~= nil then
		self:setState("dirLeft",streamReadBool(streamId),true);
		self:setState("dirRight",streamReadBool(streamId),true);
		self:setState("warnlights",streamReadBool(streamId),true);
	end;
	if self.B3.parkLeft ~= nil and self.B3.parkRight ~= nil then
		self:setState("parkLeft",streamReadBool(streamId),true);
		self:setState("parkRight",streamReadBool(streamId),true);
	end;
        --BEL3 ende--
end;

function Axion850:writeStream(streamId, connection)

        --ESLimiter anfang--
	for _,percentage in pairs(self.ESLimiter.percentage) do
		streamWriteInt8(streamId, percentage);
	end;
        --ESLimiter ende--

        --Oper anfang--
	streamWriteFloat32(streamId, self.operatingTime);
        --Oper ende--

        --toggle anfang--
	for nr,part in ipairs(self.animParts) do
		streamWriteBool(streamId,part.toMax);
	end;
        --toggle ende--

        --Twin anfang--
	streamWriteBool(streamId, self.twinWheelsActive);
        --Twin ende--

        --BEL3 anfang--
	if self.B3.work ~= nil then
		for k,v in pairs(self.B3.work) do
			streamWriteBool(streamId, v.a);
		end;
	end;
	if self.B3.highbeam ~= nil then
		streamWriteBool(streamId, self.B3.highbeam[1].a);
	end;
	if self.B3.dirLeft ~= nil and self.B3.dirRight ~= nil then
		streamWriteBool(streamId,self.B3.dirLeft[1].a);
		streamWriteBool(streamId,self.B3.dirRight[1].a);
		streamWriteBool(streamId,self.B3.wl);
	end;
	if self.B3.parkLeft ~= nil and self.B3.parkRight ~= nil then
		streamWriteBool(streamId,self.B3.parkLeft[1].a);
		streamWriteBool(streamId,self.B3.parkRight[1].a);
	end;
        --BEL3 ende--
end;

function Axion850:update(dt)

      --Dynamic anfang--
	if self:getIsActive() then
		if self.exhaustingSystem.cap ~= nil then
			local angle = math.rad(math.random(-20,5)) + self.exhaustingSystem.maxRot * self.motor.lastMotorRpm / self.exhaustingSystem.maxRpm;	
			angle = math.max(math.min(angle, 0), self.exhaustingSystem.maxRot);		
			setRotation(self.exhaustingSystem.cap, 0,0,angle);
		end;
		
		if self.time <= self.playMotorSoundTime then
			local time = (self.exhaustingSystem.deltaTime - (self.playMotorSoundTime - self.time)) / self.exhaustingSystem.deltaTime;
			local x1,y1,z1,w1 = self.exhaustingSystem.startSequence:get(time);
			local values = self.exhaustingSystem.lastValue;
			if math.abs(values.x - x1) > 0.01 or math.abs(values.y - y1) > 0.01 or math.abs(values.z - z1) > 0.01 or math.abs(values.w - w1) > 0.01 then
				setShaderParameter(self.exhaustParticleSystems[1].shape, self.exhaustingSystem.param, x1, y1, z1, w1, false);
				self.exhaustingSystem.lastValue = {x=x1, y=y1, z=z1, w=w1};
			end;
		else
			local alpha = ((self.exhaustingSystem.maxAlpha-self.exhaustingSystem.minAlpha) * self.motor.lastMotorRpm / self.exhaustingSystem.maxRpm)+self.exhaustingSystem.minAlpha;
			if math.abs(self.exhaustingSystem.lastValue.w - alpha) > 0.01 then
				local values = self.exhaustingSystem.lastValue;
				setShaderParameter(self.exhaustParticleSystems[1].shape, self.exhaustingSystem.param, values.x, values.y, values.z, alpha, false);
				self.exhaustingSystem.lastValue.w = alpha;
			end;
		end;
	end;
      --Dynamic ende--

      --ESLimiter anfang--
	if self:getIsActive() then
		if self:getIsActiveForInput() then
			local currentLevel = self.motor.speedLevel;
			if self.movingDirection < 0 then
				currentLevel = 1;
			end;

			if currentLevel ~= 0 then
				local percentage = self.ESLimiter.percentage;
				if InputBinding.isPressed(InputBinding.ESLimiter_PLUS) and percentage[currentLevel] < 100 then
					self.lastButtonDown = InputBinding.ESLimiter_PLUS;
					percentage[currentLevel] = percentage[currentLevel] + 1;
					-- don't send event to other players
					self:setNewLimit(currentLevel, percentage[currentLevel], false, true);
				elseif InputBinding.isPressed(InputBinding.ESLimiter_MINUS) and percentage[currentLevel] > 10 then
					percentage[currentLevel] = percentage[currentLevel] - 1;
					self.lastButtonDown = InputBinding.ESLimiter_MINUS;
					-- don't send event to other players
					self:setNewLimit(currentLevel, percentage[currentLevel], false, true);
				else
					if self.lastButtonDown ~= nil then
						-- only send event to other players if key was released -> performance issue
						self:setNewLimit(currentLevel, percentage[currentLevel], false);
						self.lastButtonDown = nil;
					end;	
				end;
				self.isLimiterActive = true;
			else
				if self.isLimiterActive then
					self:setNewLimit(3, 100, true);
					self.isLimiterActive = false;
				end;
				self.motor.maxRpm[3] = self.ESLimiter.maxRPM[3];
			end;
		end;
	end;
        --ESLimiter ende--

        --Help anfang--
	if self:getIsActiveForInput() then
		if InputBinding.hasEvent(InputBinding.TOGGLE_SHOW_HELP) then
			self.showHelp = not self.showHelp;
		end;
	end;
        --Help ende--

        --Lightbar anfang--
	local f,c = self:getAttachedTrailersFillLevelAndCapacity();
if f and c ~= nil then
	local prozent = f/c*100;
	if prozent >= 4.2 then
		setVisibility(self.lamps[1], true)
	else
		setVisibility(self.lamps[1], false)
	end;
	if prozent >= 8.4 then
		setVisibility(self.lamps[2], true)
	else
		setVisibility(self.lamps[2], false)
	end;
	if prozent >= 12.6 then
		setVisibility(self.lamps[3], true)
	else
		setVisibility(self.lamps[3], false)
	end;
	if prozent >= 16.8 then
		setVisibility(self.lamps[4], true)
	else
		setVisibility(self.lamps[4], false)
	end;
	if prozent >= 21 then
		setVisibility(self.lamps[5], true)
	else
		setVisibility(self.lamps[5], false)
	end;
	if prozent >= 25.2 then
		setVisibility(self.lamps[6], true)
	else
		setVisibility(self.lamps[6], false)
	end;
	if prozent >= 29.4 then
		setVisibility(self.lamps[7], true)
	else
		setVisibility(self.lamps[7], false)
	end;
	if prozent >= 33.6 then
		setVisibility(self.lamps[8], true)
	else
		setVisibility(self.lamps[8], false)
	end;
	if prozent >= 37.8 then
		setVisibility(self.lamps[9], true)
	else
		setVisibility(self.lamps[9], false)
	end;
	if prozent >= 42 then
		setVisibility(self.lamps[10], true)
	else
		setVisibility(self.lamps[10], false)
	end;
	if prozent >= 46.2 then
		setVisibility(self.lamps[11], true)
	else
		setVisibility(self.lamps[11], false)
	end;
	if prozent >= 50 then
		setVisibility(self.lamps[12], true)
	else
		setVisibility(self.lamps[12], false)
	end;
	if prozent >= 54.2 then
		setVisibility(self.lamps[13], true)
	else
		setVisibility(self.lamps[13], false)
	end;
	if prozent >= 58.4 then
		setVisibility(self.lamps[14], true)
	else
		setVisibility(self.lamps[14], false)
	end;
	if prozent >= 62.6 then
		setVisibility(self.lamps[15], true)
	else
		setVisibility(self.lamps[15], false)
	end;
	if prozent >= 66.8 then
		setVisibility(self.lamps[16], true)
	else
		setVisibility(self.lamps[16], false)
	end;
	if prozent >= 71 then
		setVisibility(self.lamps[17], true)
	else
		setVisibility(self.lamps[17], false)
	end;
	if prozent >= 75.2 then
		setVisibility(self.lamps[18], true)
	else
		setVisibility(self.lamps[18], false)
	end;
	if prozent >= 79.4 then
		setVisibility(self.lamps[19], true)
	else
		setVisibility(self.lamps[19], false)
	end;
	if prozent >= 83.6 then
		setVisibility(self.lamps[20], true)
	else
		setVisibility(self.lamps[20], false)
	end;
	if prozent >= 87.8 then
		setVisibility(self.lamps[21], true)
	else
		setVisibility(self.lamps[21], false)
	end;
	if prozent >= 92 then
		setVisibility(self.lamps[22], true)
	else
		setVisibility(self.lamps[22], false)
	end;
	if prozent >= 96.2 then
		setVisibility(self.lamps[23], true)
	else
		setVisibility(self.lamps[23], false)
	end;
	if prozent >= 100 then
		setVisibility(self.lamps[24], true)
	else
		setVisibility(self.lamps[24], false)
	end;
end;
        --Lightbar ende--

        --toggle anfang--
	if self:getIsActiveForInput() then
		for nr,part in ipairs(self.animParts) do
			if not part.autoReturn then
				if InputBinding.hasEvent(InputBinding[part.id]) then
					self:toggle(nr,not part.toMax); 
				end;
			else
				if InputBinding.isPressed(InputBinding[part.id]) then
					if not part.toMax then
						self:toggle(nr,true); 
					end;
				else
					if part.toMax then
						self:toggle(nr,false); 
					end;
				end;
			end;
		end;
	end;
	if self:getIsActive() then
		for _,part in ipairs(self.animParts) do
			if part.move then
				part.move = false;
				if part.rotMin ~= nil then
					local curRot = {getRotation(part.index)};
					local newRot = Utils.getMovedLimitedValues(curRot, part.rotMax, part.rotMin, 3, part.moveTime, dt, not part.toMax);
					setRotation(part.index, unpack(newRot));
					for i=1,3 do
						if math.abs(newRot[i]-curRot[i]) > 0.001 then
							part.move = true;
						end;
					end;
				end;
				if part.transMin ~= nil then
					local curTrans = {getTranslation(part.index)};
					local newTrans = Utils.getMovedLimitedValues(curTrans, part.transMax, part.transMin, 3, part.moveTime, dt, not part.toMax);
					setTranslation(part.index, unpack(newTrans));
					for i=1,3 do
						if math.abs(newTrans[i]-curTrans[i]) > 0.001 then
							part.move = true;
						end;
					end;
				end;
				if part.scaleMin ~= nil then
					local curScale = {getScale(part.index)};
					local newScale = Utils.getMovedLimitedValues(curScale, part.scaleMax, part.scaleMin, 3, part.moveTime, dt, not part.toMax);
					setScale(part.index, unpack(newScale));
					for i=1,3 do
						if math.abs(newScale[i]-curScale[i]) > 0.001 then
							part.move = true;
						end;
					end;
				end;
			end;
		end;
	end;
        --toggle ende--

        --Zylinder anfang--
		for i=1,table.getn(self.movingParts) do
			Cylindered.updateMovingPart(self, self.movingParts[i]);
		end;
        --Zylinder ende--

        --Twin anfang--
	if self.isEntered and InputBinding.hasEvent(InputBinding.AXIONTWINS) then
		self:settwinWheels(not self.twinWheelsActive);
	end;
	if self.twinWheelsActive then
		for i=1, self.numTwinWheels do
			local twinWheel = self.twinWheels[i];
			setVisibility(twinWheel, self.twinWheelsActive);
		end;
	else
		for i=1, self.numTwinWheels do
			local twinWheel = self.twinWheels[i];
			setVisibility(twinWheel, self.twinWheelsActive, false);
		end;
	end;
        --Twin ende--

        --BEL3 anfang--
	local updateState = {};
	if self:getIsActiveForInput() then
		if self.B3.work ~= nil then
			for i=1,table.getn(self.B3.work) do
				local wl = self.B3.work[i];
				if InputBinding.hasEvent(InputBinding[wl.na]) and InputBinding[wl.na] ~= nil and wl.na ~= nil then
					self:setState("work:"..i,not wl.a);
				end;
			end;
		end;
		if self.B3.highbeam ~= nil then
			local hb = self.B3.highbeam[1];
			if InputBinding.isPressed(InputBinding.TOGGLE_LIGHTS) then
				if hb.cnt == nil then
					hb.cnt = 0;
					if not self.lightsActive then
						self.B3.setLA = true;
						self:setLightsVisibility(true);
					end;
				else
					if hb.blk == nil then 
						hb.cnt = hb.cnt + dt;
						self.noRL = not hb.a;
					end;
				end;
				if hb.cnt > 1500 and hb.blk == nil then
					self:setState("highbeam", not hb.a);
					self.B3.setLA = false;
					self.noRL = not hb.a;
					hb.blk = true;
				end;
			else
				if hb.cnt ~= nil and hb.blk == nil then
					if self.B3.setLA then
						hb.a = false;
						self.B3.setLA = false;
						self:setLightsVisibility(false);
						self:setState("highbeam", false);
					else
						self.B3.setLA = false;
					end;
				end;
				hb.cnt = nil;
				hb.blk = nil;
			end;
		end;
		
		if self.B3.dirLeft ~=nil and self.B3.dirRight ~= nil then
			local dl = self.B3.dirLeft[1]; local dr = self.B3.dirRight[1];
			if not self.B3.wl then
				if InputBinding.hasEvent(InputBinding.BEL3LEFT) then
					self:setState("dirLeft",not dl.a);
					self:setState("dirRight", false);
				end;
				if InputBinding.hasEvent(InputBinding.BEL3RIGHT) then
					self:setState("dirLeft", false);
					self:setState("dirRight", not dr.a);
				end;
			end;
			if InputBinding.hasEvent(InputBinding.BEL3WARN) then
				self:setState("warnlights",not self.B3.wl);
				self:setState("dirLeft",self.B3.wl);
				self:setState("dirRight",self.B3.wl);
			end;
		end;
	end;
	
	if self.isServer and self:getIsActive() then
		if self.B3.brake ~= nil and self.B3.iSt then
			local iB = math.abs(Utils.sign(self.lastAcceleration)+ (-self.movingDirection*self.B3.md)) == 2;
			if self.B3.brake[1].a ~= iB then
				self:setState("brake",iB);
			end;
		end;
		if self.B3.reverse ~= nil then
			local iR = (self.movingDirection*self.B3.md) < 0;
			if self.B3.reverse[1].a ~= iR then 		
				self:setState("reverse",iR);
			end;
		end;
	end;
        --BEL3 ende--

end;

function Axion850:settwinWheels(state, noSendEvent)
	self.twinWheelsActive = state;
	if self.twinWheelsActive then
		for i=1, self.numTwinWheels do
			local twinWheel = self.twinWheels[i];
			setVisibility(twinWheel, self.twinWheelsActive);
		end;
	else
		for i=1, self.numTwinWheels do
			local twinWheel = self.twinWheels[i];
			setVisibility(twinWheel, self.twinWheelsActive, false);
		end;
	end;	
	if noSendEvent ~= true then
		Axion850WheelsEvent.sendEvent(self, Axion850WheelsEventType.TwinWheels, self.twinWheelsActive);
	end;
end;

function Axion850:startMotor()
	self.exhaustingSystem.deltaTime = self.playMotorSoundTime - self.time - self.exhaustingSystem.offset;
end;

function Axion850:updateTick(dt)

        --Wiper anfang--
		if not self.finishWiper then
			if getAnimTrackTime(self.wiperAnimCharSet, 0) % self.wiperAnimDuration <= 100 then
				setAnimTrackTime(self.wiperAnimCharSet, 0, 0.0);
				disableAnimTrack(self.wiperAnimCharSet, 0);			
				self.finishWiper = true;
			end;			
		end;
		if g_currentMission.environment.lastRainScale > 0.1 and g_currentMission.environment.timeSinceLastRain < 30 then
			if not self.isWiperActive then
				enableAnimTrack(self.wiperAnimCharSet, 0);
				self.isWiperActive = true;
			end;
		else
			if self.isWiperActive then
				self.isWiperActive = false;
				self.finishWiper = false;
			end;
		end;
		
		for _, part in pairs(self.movingParts) do
			part.isDirty = true;
		end;
         --Wipper ende--

         --Oper anfang--
    if self:getIsActive() or self.isHired then 
		self:addOperatingTime(dt);
	end;
         --Oper ende--

        --BEL3 anfang--
	if self.B3.dirLeft ~=nil and self.B3.dirRight ~= nil then
		local dl = self.B3.dirLeft; local dr = self.B3.dirRight;
		if dl[1].a or dr[1].a then
			if self.B3.blt > self.B3.bspd then
				self.B3.bls = not self.B3.bls;
				self.B3.blt = 0;
				if dl[1].a then
					for _,b in pairs(dl) do
						setVisibility(b.index,not self.B3.bls);
					end;
				end;
				if dr[1].a then
					for _,b in pairs(dr) do
						setVisibility(b.index,not self.B3.bls);
					end;
				end;
			else
				self.B3.blt = self.B3.blt+dt;
			end;
		else
			self.B3.blt = 0;
			if self.B3.bls == true then
				for _,b in pairs(dl) do
					setVisibility(b.index,false);
				end;
				for _,b in pairs(dr) do
					setVisibility(b.index,false);
				end;
				self.B3.bls = false;
			end;
		end;
	end;
	
	if self.B3.highbeam and self.realLightsActive then
		if self.B3.highbeam[1].a and not self.noRL then
			for _,light in pairs(self.lights) do
				setVisibility(light,false);
			end;
			self.noRL = true;
		elseif not self.B3.highbeam[1].a and self.noRL then
			for _,light in pairs(self.lights) do
				setVisibility(light,true);
			end;
			self.noRL = false;
		end;
	end;

	if self:getIsActive() and not self.B3.iSt and self.B3.atv ~= nil then
		local atvb = false;
		if self.B3.atv.B3 ~= nil then 
			local atv = self.B3.atv.B3;
			if atv.brake ~= nil and self.B3.brake ~= nil then
				atvb = true;
				if self.B3.brake[1].a ~= atv.brake[1].a then
					self:setState("brake",atv.brake[1].a,true);
				end;
			end;
			if atv.reverse ~= nil and self.B3.reverse ~= nil then
				if self.B3.reverse[1].a ~= atv.reverse[1].a then
					self:setState("reverse",atv.reverse[1].a,true);
				end;
			end;
		end;
		if not atvb and self.B3.brake ~= nil then
			local iB = math.abs(Utils.sign(self.B3.atv.lastAcceleration)+ (-self.movingDirection*self.B3.md)) == 2;
			if self.B3.brake[1].a ~= iB then
				self:setState("brake",iB);
			end;
		end;
	end;
        --BEL3 ende--
		
		--Hydraulic anfang--
    if self:getIsActive() then
        if self.backHydraulic ~= nil then
			local x, y, z = getRotation(self.backHydraulic.bottomArm);
			setRotation(self.backHydraulic.rootArm, x, y, z);
		end;
	end;
		--Hydraulic ende--
end;

function Axion850:draw()

      --ESLimiter anfang--
	local currentLevel = self.motor.speedLevel;
	local text = "Deaktiviert";
	if self.movingDirection < 0 then
		currentLevel = 1;
	end;
	if currentLevel ~= 0 then
		local percentage = tonumber(string.format("%d", self.ESLimiter.percentage[currentLevel]));
		if percentage < 100 then
			percentage = " " .. percentage;
		end;
		text = g_i18n:getText("ESLimiter_level") .. currentLevel .. ":" .. percentage .. "% ";	
	end;
	self.eslOverlay:render();
	setTextBold(true);
	setTextAlignment(RenderText.ALIGN_LEFT);
	renderText(self.ESLimiter.xPos + 0.13, self.ESLimiter.yPos + 0.0085, 0.02, text);
      --ESLimiter ende--

      --Oper anfang--
	local operatingHours = math.floor((self.operatingTime / 1000 / 60 / 60)*10) / 10;
	local hours = math.floor(operatingHours);
	local minutes = tostring(math.floor((operatingHours - hours)*10));
	
	self.ohOverlay:render();
	setTextAlignment(RenderText.ALIGN_RIGHT);
	setTextBold(true);
	setTextColor(1,1,1,1);	
	renderText(self.ohX+0.062, self.ohY+self.yOffset, 0.022, tostring(hours) .. ",");
	setTextColor(0.95,0,0,1);
	renderText(self.ohX+0.071, self.ohY+self.yOffset, 0.023, minutes);
	setTextColor(1,1,1,1);
	renderText(self.ohX+0.08, self.ohY+self.yOffset, 0.023, "h");
	setTextAlignment(RenderText.ALIGN_LEFT);
      --Oper ende--

      --Help anfang--
	if not self.showHelp then
		renderText(0.74, 0.22, 0.02, "Taste Numpad 0: Info Einschalten");
	 else
		-- Author Repi @MRA-Modding
		renderText(0.74, 0.22, 0.02, "Taste Numpad 0: Info Abschalten");
		renderText(0.68, 0.86, 0.025, "- Axion 850 Info -");			
		renderText(0.68, 0.83, 0.02, "Taste Pos1     : Rundumleuchte ein/aus");
		renderText(0.68, 0.81, 0.02, "Taste Num 1    : Blinker links");
		renderText(0.68, 0.79, 0.02, "Taste Num 2    : Warnblinker");			
		renderText(0.68, 0.77, 0.02, "Taste Num 3    : Blinker rechts");
		renderText(0.68, 0.75, 0.02, "Taste Num 4    : A-Licht front an/aus");
		renderText(0.68, 0.73, 0.02, "Taste Num 5    : A-Licht back an/aus");
		renderText(0.68, 0.71, 0.02, "Taste Num 6    : Dach auf/zu");
		renderText(0.68, 0.69, 0.02, "Taste Num 7    : Heckfenster auf/zu");
		renderText(0.68, 0.67, 0.02, "Taste Num 8    : Tuer links auf/zu");
		renderText(0.68, 0.65, 0.02, "Taste Num 9    : Tuer rechts auf/zu");
		renderText(0.68, 0.63, 0.02, "Taste Num Enter: RUL hoch/runter");
		renderText(0.68, 0.61, 0.02, "Taste Bild auf : ESLimiter erhoehen");
		renderText(0.68, 0.59, 0.02, "Taste Bild ab  : ESLimiter verringern");
                renderText(0.68, 0.57, 0.02, "Taste 5        : Twins All ein/aus");
                renderText(0.68, 0.49, 0.02, "by Repi @mra-modding.de");
	end;
      --Help ende--

      --toggle anfang--
	if self:getIsActiveForInput() then
		for _,part in ipairs(self.animParts) do
			if part.helpText then
			end;
		end;
	end;
      --toggle ende--

        --BEL3 anfang--
	if self.B3.work ~= nil then
		for i=1,table.getn(self.B3.work) do
			local wl = self.B3.work[i];
			if wl.ht ~= nil and wl.na ~= nil then
    	end;
    end;
  end;
        --BEL3 ende--
end;

function Axion850:onAttach()
	local atv = self.attacherVehicle;
	while true do 
		if atv.attacherVehicle ~= nil then
			atv = atv.attacherVehicle;
		else 
			break;
		end;
	end;
	self.B3.atv = atv;
end;

function Axion850:onDetach()
	self.B3.atv = nil;
end;

function Axion850:toggle(id,state,nes)
	Axion850AnimEvent.sendEvent(self,id,state,nes);
	self.animParts[id].toMax = state;
	self.animParts[id].move = true;
end;

function Axion850:attachImplement(implement)

	local jointType = implement.object.attacherJoint.jointType;
	local jointIndex = implement.jointDescIndex;

	if jointType == Vehicle.JOINTTYPE_IMPLEMENT then
		if jointIndex == 2 then
			setVisibility(self.attacherJoints[jointIndex].topArm.rotationNode, true);
		end;
	end;
	
	self.updateJoint = true;
end;

function Axion850:detachImplement(implementIndex)

	local implement = self.attachedImplements[implementIndex];
	local jointIndex = implement.jointDescIndex;

	if jointIndex == 2 then
		setVisibility(self.attacherJoints[jointIndex].topArm.rotationNode, false);
	end;
end;

function Axion850:validateAttacherJoint(implement, jointDesc, dt)
    if self.updateJoint then
        self.updateJoint = false;
        return true;
    end;
    return false;
end;

function Axion850:detachImplement(implement)
	for i = 1,25 do
		setVisibility(self.lamps[i], false)
	end;
end;


function Axion850:addOperatingTime(addTime)
	
	local time = math.max(Utils.getNoNil(addTime, 0), 0);	
	self.operatingTime = self.operatingTime + time;	
	
	if self.operatingTime > (99999.9 * 1000 * 60 * 60) then
		self.operatingTime = 0;
	end;
end;

function Axion850:setNewLimit(level, percentage, doRestoreLimit, noEventSend)	
	Axion850Event.sendEvent(self, level, percentage, doRestoreLimit, noEventSend);	
	local currentRPM = self.ESLimiter.maxRPM;
	if not doRestoreLimit then
		self.ESLimiter.percentage[level] = percentage;
		if self.movingDirection > 0 then
			self.motor.maxRpm[level] = tonumber(string.format("%d",(percentage * currentRPM[level] / 100)));
		else
			self.motor.maxRpm[3] = tonumber(string.format("%d",(percentage * currentRPM[level] / 100)));
		end;
	else
		self.motor.maxRpm[3] = self.ESLimiter.maxRPM[3];
	end;
end;

function Axion850:loadFromAttributesAndNodes(xmlFile, key, resetVehicles)
	if not resetVehicles then
		local valueStr = getXMLString(xmlFile, key.."#limiterValues");		
		if valueStr ~= nil then
			local found = string.find(valueStr, ";");
			local index = 1;
			while found ~= nil do
				local value = string.sub(valueStr, 0, found-1);
				valueStr = string.sub(valueStr, found+1);
				found = string.find(valueStr, ";");
				self.ESLimiter.percentage[index] = tonumber(value);
				if found == nil then
					self.ESLimiter.percentage[index+1] = tonumber(valueStr);
				end;
				index = index + 1;	
			end;
		end;
	end;

       --Oper anfang--
	if not resetVehicles then
		local operatingTime = Utils.getNoNil(getXMLFloat(xmlFile, key .. "#operatingTime"), self.operatingTime) * 1000;
		self:addOperatingTime(operatingTime);
	end;
       --Oper ende--

    return BaseMission.VEHICLE_LOAD_OK;
end;

function Axion850:getSaveAttributesAndNodes(nodeIdent)
	local values = "";	
	for k,v in pairs(self.ESLimiter.percentage) do
		if k ~= 1 then
			values = values .. ";";
		end;
		values = values .. string.format("%d", v);
	end;
	
    local attributes = 'limiterValues="'.. values .. '"';

    local attributes = 'operatingTime="' .. tostring((self.operatingTime / 1000)) ..'"';
    return attributes, nil;
end;			

function Axion850:onLeave()

      --Dynamic anfang--
	if self.exhaustingSystem.cap ~= nil then
		setRotation(self.exhaustingSystem.cap, 0,0,0);
	end;
      --Dynamic ende--

      --Wiper anfang--
	if self.isWiperActive then
		disableAnimTrack(self.wiperAnimCharSet, 0);	
	end;
      --Wiper ende--

	self.showHelp = false;

        --BEL3 anfang--
	if self.B3.left then
		self.B3.left = false;
		return;
	else
		self.B3.left = true;
	end;
	if self.B3.work ~= nil then
		for i=1,table.getn(self.B3.work) do
			local wl = self.B3.work[i];
			local newstate = false;
			if wl.so then
				newstate = wl.a;
			end;
			self:setState("work:"..i,newstate,true);
		end;
	end;

	if self.B3.highbeam ~= nil then
		self:setState("highbeam", false,true);
	end;

	if self.B3.lso then
		--koronas an , licht aus
		local la = self.lightsActive;
		if la then
			self:setLightsVisibility(false);
			if self.B3.parkLeft and self.B3.parkRight then
				self:setState("parkLeft",true);
				self:setState("parkRight",true);
			else
        for _, corona in pairs(self.lightCoronas) do
            setVisibility(corona, true);
        end;
      end;
    end;
	end;

	if not self.B3.wl then
		if self.B3.dirLeft and self.B3.dirRight then
			if self.B3.parkLeft and self.B3.parkRight then
				if self.B3.dirLeft[1].a then
					self:setState("parkLeft",true);
				end;
				if self.B3.dirRight[1].a then
					self:setState("parkRight",true);
				end;
			end;
			if self.B3.dirLeft[1].a then
				self:setState("dirLeft",false);
			end;
			if self.B3.dirRight[1].a then
				self:setState("dirRight",false);
			end;
		end;
	end;
        --BEL3 ende--
end;	

function Axion850:onEnter()

       --Wiper anfang--
	if self.isWiperActive then
		enableAnimTrack(self.wiperAnimCharSet, 0);	
	end;
       --Wiper ende--

        --BEL3 anfang--
	self.B3.left = false;
	if self.B3.work then
		for i=1,table.getn(self.B3.work) do
			local wl = self.B3.work[i];
			self:setState("work:"..i,wl.a,true);
		end;
	end;
	if self.B3.parkLeft and self.B3.parkRight then
		if self.B3.parkLeft[1].a and self.B3.parkRight[1].a then
			self:setState("parkLeft",false);
			self:setState("parkRight",false);
			self:setLightsVisibility(true);
		end;
	end;
	if self.B3.dirLeft and self.B3.parkLeft then
		if self.B3.parkLeft[1].a then
			self:setState("dirLeft",true);
			self:setState("parkLeft",false);
		end;
	end;
	if self.B3.dirRight and self.B3.parkRight then
		if self.B3.parkRight[1].a then
			self:setState("dirRight",true);
			self:setState("parkRight",false);
		end;
	end;
        --BEL3 ende--
end;

function Axion850:setState(object,state,noEventSend)
	if object=="warnlights" then
		self.B3.wl = state;
		Axion850LightStateEvent.sendEvent(self,object,state,noEventSend);
		return;
	end;
	local o,c = unpack(Utils.splitString(":",object));
	local lt = self.B3[o];
	local cs = 1; local cm = table.getn(lt);
	if c ~= nil then
		cs = tonumber(c); cm = cs;
	end;
	for c=cs,cm do
		local l = lt[c];
		l.a = state;
		setVisibility(l.index,state);
		local rstate = false; 
		local bstate = false;
		if self.isEntered then
			rstate = state;
		else
			bstate = state;
		end;
		if l.rl ~= nil then
			setVisibility(l.rl,rstate);
		end;
		if l.fl ~= nil then
			setVisibility(l.fl,bstate);
		end;
	end;
	Axion850LightStateEvent.sendEvent(self,object,state,noEventSend);
end;